package com.hiramine.fileselectiondialogmultiselectsample;

import java.util.List;
import java.util.ListIterator;

import android.content.Context;
import android.graphics.Color;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

public class FileInfoArrayAdapter extends ArrayAdapter<FileInfo>
{
	private List<FileInfo>	m_listFileInfo; // ファイル情報リスト

	// コンストラクタ
	public FileInfoArrayAdapter(	Context context,
									List<FileInfo> objects )
	{
		super( context, -1, objects );

		m_listFileInfo = objects;
	}

	// m_listFileInfoの一要素の取得
	@Override
	public FileInfo getItem( int position )
	{
		return m_listFileInfo.get( position );
	}

	// 一番上の選択されたアイテムの位置の取得
	public int getFirstSelectedItemPosition()
	{
		ListIterator<FileInfo> it = m_listFileInfo.listIterator();
		while( it.hasNext() )
		{
			FileInfo fileinfo = it.next();
			if( fileinfo.isSelected() )
			{
				return it.nextIndex() - 1;
			}
		}
		return -1;
	}

	// positionで指定したアイテムの次の選択されたアイテムの位置の取得
	public int getNextSelectedItemPosition( int iPosition )
	{
		ListIterator<FileInfo> it = m_listFileInfo.listIterator( iPosition );
		// 一つ進める
		it.next();
		while( it.hasNext() )
		{
			FileInfo fileinfo = it.next();
			if( fileinfo.isSelected() )
			{
				return it.nextIndex() - 1;
			}
		}
		return -1;
	}

	// 一要素のビューの生成
	@Override
	public View getView(	int position,
							View convertView,
							ViewGroup parent )
	{
		// レイアウトの生成
		if( null == convertView )
		{
			Context context = getContext();
			// レイアウト
			LinearLayout layout = new LinearLayout( context );
			layout.setPadding( 10, 10, 10, 10 );
			layout.setBackgroundColor( Color.WHITE );
			convertView = layout;
			// テキスト
			TextView textview = new TextView( context );
			textview.setTag( "text" );
			textview.setTextColor( Color.BLACK );
			textview.setPadding( 10, 10, 10, 10 );
			layout.addView( textview );
		}

		// 値の指定
		FileInfo fileinfo = m_listFileInfo.get( position );
		TextView textview = (TextView)convertView.findViewWithTag( "text" );
		if( fileinfo.getFile().isDirectory() )
		{ // ディレクトリの場合は、名前の後ろに「/」を付ける
			textview.setText( fileinfo.getName() + "/" );
		}
		else
		{
			textview.setText( fileinfo.getName() );
		}

		// 選択行と非選択行
		if( fileinfo.isSelected() )
		{ // 選択行：青背景、白文字
			convertView.setBackgroundColor( Color.BLUE );
			textview.setTextColor( Color.WHITE );
		}
		else
		{ // 非選択行：白背景、黒文字
			convertView.setBackgroundColor( Color.WHITE );
			textview.setTextColor( Color.BLACK );
		}

		return convertView;
	}
}
