package com.hiramine.fileselectiondialogmultiselectsample;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.Context;
import android.content.DialogInterface;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

public class FileSelectionDialog
		implements OnItemClickListener, OnClickListener
{
	private Context					m_parent;				// 呼び出し元
	private OnFileSelectListener	m_listener;			// 結果受取先
	private AlertDialog				m_dlg;					// ダイアログ
	private FileInfoArrayAdapter	m_fileinfoarrayadapter; // ファイル情報配列アダプタ
	private String[]				m_astrExt;				// フィルタ拡張子配列
	private Button					m_buttonOK;			// 「OK」ボタン

	// コンストラクタ
	public FileSelectionDialog( Context context,
								OnFileSelectListener listener,
								String strExt )
	{
		m_parent = context;
		m_listener = (OnFileSelectListener)listener;

		// 拡張子フィルタ
		if( null != strExt )
		{
			StringTokenizer tokenizer = new StringTokenizer( strExt, "; " );
			int iCountToken = 0;
			while( tokenizer.hasMoreTokens() )
			{
				tokenizer.nextToken();
				iCountToken++;
			}
			if( 0 != iCountToken )
			{
				m_astrExt = new String[iCountToken];
				tokenizer = new StringTokenizer( strExt, "; " );
				iCountToken = 0;
				while( tokenizer.hasMoreTokens() )
				{
					m_astrExt[iCountToken] = tokenizer.nextToken();
					iCountToken++;
				}
			}
		}
	}

	// ダイアログの作成と表示
	public void show( File fileDirectory )
	{
		// タイトル
		String strTitle = fileDirectory.getAbsolutePath();

		// リストビュー
		ListView listview = new ListView( m_parent );
		listview.setScrollingCacheEnabled( false );
		listview.setOnItemClickListener( this );
		// ファイルリスト
		File[] aFile = fileDirectory.listFiles( getFileFilter() );
		List<FileInfo> listFileInfo = new ArrayList<FileInfo>();
		if( null != aFile )
		{
			for( File fileTemp : aFile )
			{
				listFileInfo.add( new FileInfo( fileTemp.getName(), fileTemp ) );
			}
			Collections.sort( listFileInfo );
		}
		// 親フォルダに戻るパスの追加
		if( null != fileDirectory.getParent() )
		{
			listFileInfo.add( 0, new FileInfo( "..", new File( fileDirectory.getParent() ) ) );
		}
		m_fileinfoarrayadapter = new FileInfoArrayAdapter( m_parent, listFileInfo );
		listview.setAdapter( m_fileinfoarrayadapter );

		Builder builder = new AlertDialog.Builder( m_parent );
		builder.setTitle( strTitle );
		builder.setPositiveButton( "OK", null );
		builder.setNegativeButton( "Cancel", null );
		builder.setView( listview );
		m_dlg = builder.show();

		// Builder#setPositiveButton() でリスナーを指定した場合、ダイアログは閉じてしまう。
		// ファイルを一つも選択していない場合は、ダイアログを閉じないようにしたいので、
		// Button#setOnClickListener() でリスナーを指定する。
		m_buttonOK = m_dlg.getButton( DialogInterface.BUTTON_POSITIVE );
		m_buttonOK.setOnClickListener( this );
	}

	// FileFilterオブジェクトの生成
	private FileFilter getFileFilter()
	{
		return new FileFilter()
		{
			public boolean accept( File arg0 )
			{
				if( null == m_astrExt )
				{ // フィルタしない
					return true;
				}
				if( arg0.isDirectory() )
				{ // ディレクトリのときは、true
					return true;
				}
				for( String strTemp : m_astrExt )
				{
					if( arg0.getName().toLowerCase().endsWith( "." + strTemp ) )
					{
						return true;
					}
				}
				return false;
			}
		};
	}

	// ListView内の項目をクリックしたときの処理
	public void onItemClick(	AdapterView<?> l,
								View v,
								int position,
								long id )
	{
		FileInfo fileinfo = m_fileinfoarrayadapter.getItem( position );

		if( true == fileinfo.getFile().isDirectory() )
		{ // ディレクトるをクリックしたときは、ダイアログを閉じて、ダイアログの再作成
			if( null != m_dlg )
			{
				m_dlg.dismiss();
				m_dlg = null;
			}
			show( fileinfo.getFile() );
		}
		else
		{ // ファイルをクリックしたときは、選択状態の反転と、クリック行の表示更新
			fileinfo.setSelected( !fileinfo.isSelected() );
			l.getAdapter().getView( position, v, l );
		}
	}

	// 「OK」ボタンをクリックしたときの処理
	public void onClick( View v )
	{
		if( v == m_buttonOK )
		{
			// ファイルが選択されているか
			int iPosition_first = m_fileinfoarrayadapter.getFirstSelectedItemPosition();
			int iPosition = iPosition_first;
			int iCount = 0;
			while( -1 != iPosition )
			{
				iPosition = m_fileinfoarrayadapter.getNextSelectedItemPosition( iPosition );
				iCount++;
			}
			if( 0 == iCount )
			{ // ひとつも選択されていない
				Toast.makeText( m_parent, "No file selected.", Toast.LENGTH_SHORT ).show();
			}
			else
			{ // ひとつ以上選択されている
				File[] aFile = new File[iCount];
				iPosition = iPosition_first;
				iCount = 0;
				while( -1 != iPosition )
				{
					aFile[iCount] = m_fileinfoarrayadapter.getItem( iPosition ).getFile();
					iPosition = m_fileinfoarrayadapter.getNextSelectedItemPosition( iPosition );
					iCount++;
				}
				m_dlg.dismiss();
				m_dlg = null;
				// リスナーのハンドラを呼び出す
				m_listener.onFileSelect( aFile );
			}
		}
	}

	// 選択したファイルの情報を取り出すためのリスナーインターフェース
	public interface OnFileSelectListener
	{
		// ファイルが選択されたときに呼び出される関数
		public void onFileSelect( File[] aFile );
	}
}
